<?php

class togmine_push_issues extends togmine_synchronizer_base {
  function __construct() {
    $this->base_entry = array(
      'duration' => 0,
      'project' => new stdClass,
      // This is for machine use only and is not translatable.
      'created_with' => 'TogMine: Autocomplete Suggestions',
      'tag_names' => array(togmine_var('autocomplete_suggestion_tag_name')),
      // @todo Minor: Set this from the Redmine issue or project?
      // 'billable' => TRUE,
    );

    $this->start = date_create('last sunday')->format(TOGGL_DATE_FORMAT);
  }

  function execute() {
    // Fetch the token.
    $token = set_active_rest_api_key('toggl');

    $result = db_query('SELECT * FROM {togmine_projects}');
    while ($row = db_fetch_object($result)) {
      foreach (redmine_api_fetch_all('issues', array('project_id' => $row->redmine_project_id)) as $issue) {
        $this->entry_save($issue, $row->toggl_project_id, $token);
      }
    }

    // Log the number of items that were synchronized.
    $this->saved = count($this->synchronized_ids);

    // Do not use the end() method, which runs only once after synchronization.
    // Handle orphans for each account when that account has synchronized.
    if (!empty($this->synchronized_ids)) {
      // SQL components to deal with orphans.
      $from = "FROM {togmine_issues} WHERE toggl_api_token = '%s' AND toggl_time_entry_id NOT IN (%s)";
      $args = array($token, implode(',', $this->synchronized_ids));

      // Quietly delete orphaned autocomplete suggestions.
      $result = db_query("SELECT toggl_time_entry_id $from", $args);
      while ($id = db_result($result)) {
        toggl_rest_api_time_entry_delete($id);
      }

      // Log the number of items that were deleted.
      $this->deleted = db_result(db_query("SELECT count(*) $from", $args));

      db_query("DELETE $from", $args);
    }
  }

  /**
   * Saves one Toggl.com Time entry for an Issue.
   */
  function entry_save($issue, $toggl_project_id, $token) {
    // Skip this issue if it has already been synchronized.
    if (isset($this->synchronized_ids[$issue->id])) {
      return;
    }

    // Load the existing time entry, if there is one.
    $sql = "SELECT toggl_time_entry_id FROM {togmine_issues} WHERE toggl_api_token = '%s' AND redmine_issue_id = %d";
    if ($id = db_result(db_query($sql, $token, $issue->id))) {
      $entry = toggl_rest_api_time_entry_load($id);
    }

    // Create a new time entry even if the old one was deleted manually in
    // Toggl.com's UI.
    if (!$entry) {
      // Type-casting $base_entry also conveniently forces it to be a copy
      // instead of a reference.
      $entry = (Object) $this->base_entry;
    }

    $entry->project->id = $toggl_project_id;
    $entry->description = togmine_redmine_issue_to_toggl_time_entry_description($issue);
    $entry->start = $this->start;
    $entry->duration = 0;
    // Reset the workspace ID to be valid for each user.
    $entry->workspace_id = $this->get_workspace_id();

    // Save the time entry to Toggl.com.
    if ($saved = toggl_rest_api_time_entry_save($entry)) {
      $this->synchronized_ids[$issue->id] = $saved->id;

      // Update the map if this is a new entry.
      if (!$id) {
        db_query("INSERT INTO {togmine_issues} (redmine_issue_id, toggl_time_entry_id, toggl_api_token) VALUES (%d, %d, '%s')", $issue->id, $saved->id, $token);
      }
    }
  }
}
